<?php

class Twitter {
	
	// Configure our basic variables
	private $tweet_count = 10;
	private $twitter_api_url = 'http://api.twitter.com/1/statuses/user_timeline.xml?screen_name=';
	
	// Define our cache properties
	private $cache_folder = './cache';
	private $cache_expiration = 3600;
	
	/***
	*
	*	The main function the invokes the other calls
	*
	***/
	
	public function getTweets($twitter_username = FALSE)
	{
		// If we've not been supplied a username, then return an error
		if(!$twitter_username) return 'Error, you have not provided a Twitter username.';
		
		// Create our full cache file path
		$cache_file = $this->cache_folder . '/' . $twitter_username . '.xml';
		
		// If the cache doesn't exist, or has expired, connect to the Twitter API and store the data...
		if(!is_file($cache_file) || (date("U")-date("U", filemtime($cache_file))) > $this->cache_expiration) {
			if(is_string($value = $this->storeCache($cache_file, $twitter_username))) return $value;
		}
	
		// By default, return the contents of our cache
		return $this->readCache($cache_file);
	}
	
	/***
	*
	*	The cache READ function
	*
	***/
	
	private function readCache($cache_file = FALSE, $tweets_array = FALSE)
	{
		// Load in the stored XML file...
		$cacheXML = simplexml_load_file($cache_file);
		
		// Make sure we have a valid XML object, after the conversion
		if(is_object($cacheXML)) {
			// Loop through each twitter status and append to our tweets array
			foreach($cacheXML->status as $status) {
				$tweets_array[] = array(
					'created' => (string)$status->created_at,
					'text' => (string)$status->text
				);
			}
			
			// If we have a valid tweets array, return it sliced by the number of tweets configured at the top, otherwise return an error.
			return (is_array($tweets_array))? array_slice($tweets_array, 0, $this->tweet_count) : 'There are no tweets available.';
		}
		
		// If our XML object failed, return the necessary error.
		else return 'Error, the cache file does not appear to be valid XML.';
	}
	
	/***
	*
	*	The cache WRITE function
	*
	***/
	
	private function storeCache($cache_file = FALSE, $twitter_username = FALSE)
	{
		// Check our cache folder exists
		if(is_dir($this->cache_folder))	{
			// Check our cache folder is writable!
			if(is_writable($this->cache_folder)) {
				// Connect to the twitter API and grab the specified users' statuses...
				$tweets = file_get_contents($this->twitter_api_url . $twitter_username);
				
				// Check the server response header from the twitter API
				switch($http_response_header[0]) {
					case 'HTTP/1.0 200 OK':
						$cacheHandle = fopen($cache_file, 'w');
						fwrite($cacheHandle, $tweets);
						fclose($cacheHandle);
						break;
					case 'HTTP/1.0 401 Unauthorized':
						return 'Error, the Twitter password you have supplied is either invalid or missing.';
						break;
					case 'HTTP/1.0 404 Not Found';
						return 'Error, the Twitter username you have supplied has not been found.';
						break;
					default:
						return 'Error, an error occurred while communicating with Twitter (' . $http_response_header[11] . '). Please try again later.';
						break;
				}
				
				// If we get this far, all went well and we didn't encounter any errors
				return TRUE;
			}
			
			// Return the appropriate error message
			else return 'Error, the folder you have specified is not writable.';
		}
		
		// Return the appropriate error message
		else return 'Error, the folder you have specified does not exist.';
	}
}
?>